package de.hdm_stuttgart.mi.sd1.task2;

/**
 * Describing a battery's state of charge categories.
 */
public enum BatteryState {

    /**
     * State of charge is below 10% of the battery's capacity.
     */
    LOW("SOC below 10 %"),
    /**
     * State of charge is in between and including 10% and 90% of the battery's capacity.
     */
    NORMAL("SOC between 10% and 90%"),
    /**
     * State of charge is above 90% of the battery's capacity.
     */
    HIGH("SOC above 90%");

    /**
     * <p>Turn the state of charge percentage into categories:</p>
     *
     * @param socPercentage State of charge as a percentage of the battery's {@link BatteryPlanner#capacity}.
     *
     * @return
     * <ul>
     *     <li> 0 &lt;= percentage &lt; 10% : return {@link #LOW}</li>
     *     <li> 10 &lt;= percentage &lt; 90% : return {@link #NORMAL}</li>
     *     <li> 90 &lt;= percentage &lt;= 100% : return {@link #HIGH}</li>
     * </ul>
     *
     */
    static public BatteryState getState(final double socPercentage) {
        if (socPercentage < 10) {
            return BatteryState.LOW;
        } else if (socPercentage <= 90) {
            return BatteryState.NORMAL;
        } else {
            return BatteryState.HIGH;
        }
    }

    final String description;

    BatteryState(final String description) {
        this.description = description;
    }
}
