package de.hdm_stuttgart.mi.sd1.task1;

/** <p>Array helper method.</p>
 *
 */
public class D_ArrayHelper {

    /**
     * <p>Gather information about an <code>int</code> array's negative, zero and positive values.</p>
     *
     * <p>An <code>int</code> array may contain negative, zero and positive values. This information is being
     * provided by a {@link String}:</p>
     *
     * <table class="goikTableDefaults">
     *   <caption>Filtering {@link String} instances</caption>
     *   <tr>
     *     <th>Values</th>
     *     <th>Result String</th>
     *   </tr>
     *   <tr>
     *       <td><code>{1, 7, 5,20}</code></td>
     *       <td><code>"positives"</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>{-1, -20, 0, -33}</code></td>
     *       <td><code>"negatives,zeroes"</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>{1, 7, -4, 5, 20, 0}</code></td>
     *       <td><code>"negatives,zeroes,positives"</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>{0}</code></td>
     *       <td><code>"zeroes"</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>{-4}</code></td>
     *       <td><code>"negatives"</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>{}</code></td>
     *       <td rowspan="2"><code>""</code></td>
     *   </tr>
     *   <tr>
     *       <td><code>null</code></td>
     *   </tr>
     *
     * </table>
     *
     * <p>The exact names and their order matters:</p>
     * <ol>
     *     <li><code>"negatives"</code></li>
     *     <li><code>"zeroes"</code></li>
     *     <li><code>"positives"</code></li>
     * </ol>
     *
     * <p>Thus we will see e.g. <code>"negatives,positives"</code> but never <code>"positives,negatives"</code>.</p>
     *
     * <section class="implementationHints">
     *     <h4 class="implementationHints">Hint:</h4>
     *     <p>Mind whitespace: <code>"negatives,<span style="color:red;">␣</span>zeroes"</code> differs from
     *     <code>"negatives,zeroes"</code> by an additional space character "<span style="color:red;">␣</span>"
     *     following the comma.</p>
     * </section>
     *
     * @param values Integer values
     * @return String Categorization of signs as being stated above or an empty string if no values being present.
     */
    public static String getSigns(final int[] values) {

        // Handle trivial cases
        //
        if (null == values || 0 == values.length) {
            return "";
        }

        // Step one: See whether negatives, zeroes or positives do show up among our values.
        //
        boolean hasNegatives = false, hasZeroes = false, hasPositives = false;

        for (final int value: values) {
            if (value < 0 ) {
                hasNegatives = true;
            } else if (0 == value) {
                hasZeroes = true;
            } else {
                hasPositives = true;
            }

            if (hasNegatives && hasZeroes && hasPositives) { // Minor optimization
                break;
            }
        }

        // Step 2: Assemble a possibly true subset of {"negatives", "zeroes", "positives"}
        //
        final String[] signs = new String[3];
        int signCount = 0;

        if (hasNegatives) {
            signs[signCount++] = "negatives";
        }

        if (hasZeroes) {
            signs[signCount++] = "zeroes";
        }

        if (hasPositives) {
            signs[signCount++] = "positives";
        }

        // Step 3: Assemble retrieved signs into a result string
        //
        final StringBuilder ret = new StringBuilder(signs[0]);// At least one must exist

        for (int i = 1; i < signCount; i++) {
            ret.append(',').append(signs[i]);
        }
        return ret.toString();
    }

    private D_ArrayHelper(){/* Suppressing constructor Javadoc HTML generation */}
}
